@@ -98,6 +98,67 @@ int brl_ser_open_port (int Port)
 	return 1;
 }
 
+/* for HandyTech devices */
+int handy_set_comm_param()
+{
+	struct termios options;
+
+	/* get the current options from the port */
+	tcgetattr (fdSerial, &options);
+
+        // options.c_cflag = 0;
+
+	/* set the baud rate to 19200 */
+	cfsetispeed (&options, B19200);
+	cfsetospeed (&options, B19200);
+
+	// options.c_cflaf |= (CLOCAL | CREAD);
+
+	/* set parity odd */
+	options.c_cflag |= PARENB;							
+	options.c_cflag |= PARODD;
+	options.c_iflag &= ~(INPCK | ISTRIP);
+	
+	/* set character size */
+	options.c_cflag &= ~CSIZE;	
+	options.c_cflag |= CS8;		
+
+
+	/* set 1 stop bits */
+	options.c_cflag &= ~CSTOPB;
+	
+	
+	/* disable flow controls */
+	options.c_cflag &= ~CRTSCTS;	                /* disable CTS/RTS flow control	 */
+	options.c_iflag &= ~(IXON | IXOFF | IXANY);	/* disable XON/XOFF flow control */
+
+
+	/* choose raw input */
+	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
+
+	/* choose raw output */
+	// options.c_oflag &= ~OPOST;	
+	options.c_oflag &= ~OLCUC;
+	options.c_oflag &= ~ONLCR;
+	options.c_oflag &= ~OCRNL;
+	// options.c_oflag &= ~NOCR;
+	/*options.c_oflag &= ~ONLRET;*/
+
+
+        /* enable the receiver and set local mode*/
+	options.c_cflag |= (CLOCAL | CREAD);
+
+	/* timeouts */
+	options.c_cc[VMIN] = 0;
+	options.c_cc[VTIME] = 10;
+
+	/* set the options for the port */
+	tcsetattr (fdSerial, TCSANOW, &options);
+
+	return 1;
+
+}
+
 /* ... */
 int brl_ser_set_comm_param (long BaudRate, char Parity, short StopBits, char FlowCtrl)
 {
@@ -265,8 +326,17 @@ int brl_ser_read_data (char *DataBuff, int MaxLen)
 {
   /*	fcntl (fdSerial, F_SETFL, FNDELAY);  */
   fcntl (fdSerial, F_SETFL, O_NONBLOCK);
+  // fcntl (fdSerial, F_SETFL, 0);
 /* read function returns 0 if no characters are available on the port */
-	return read (fdSerial, DataBuff, MaxLen);
+
+  /*r = read (fdSerial, DataBuff, MaxLen);
+
+  for (i=0; i < r; i++)
+	  if ((DataBuff[i] > 0x00) && (DataBuff[i] < 0x10)) fprintf(stderr, "##%0x##", DataBuff[i]);
+
+
+	  return r;*/
+  return read (fdSerial, DataBuff, MaxLen);
 }
 
 /* ... */
