@@ -108,7 +108,8 @@ CmdFctType cmd_key_cmds[] =
 	{"L00K14", {"goto last",	 	NULL}},	
 
 	{"L01K12", {"goto caret",	 	NULL}},	
-	{"L01K01", {"attributes at caret", 	NULL}},	
+	{"L01K01", {"attributes at caret", 	NULL}},
+	{"L01K02", {"watch current object", 	NULL}},
 
 	{"L03K05", {"flat review",	 	NULL}},	
 	{"L03K04", {"window hierarchy",	 	NULL}},
@@ -135,6 +136,12 @@ CmdFctType cmd_key_cmds[] =
 	{"L09K03", {"char right", 		NULL}},
 	{"L09K04", {"display left", 		NULL}},
 	{"L09K06", {"display right", 		NULL}},
+	
+	{"L10K01", {"braille on/off",		NULL}},
+	{"L10K02", {"speech on/off",		NULL}},
+	{"L10K03", {"magnifier on/off",		NULL}},
+	{"L10K04", {"braille monitor on/off",	NULL}},
+	
 	{NULL,	   {NULL}}
 };
     
