@@ -26,6 +26,7 @@
 #include <string.h>
 #include <glib.h>
 #include <SRObject.h>
+#include <SRLow.h>
 #include <gtk/gtk.h>
 #include "srmag.h"
 #include "srspc.h"
@@ -1210,6 +1211,23 @@ src_caret_attributes ()
     return rv;
 }
 
+static gboolean
+src_watch_current ()
+{
+    gboolean rv = FALSE;
+
+    if (src_crt_sro)
+	srl_set_watch_for_object (src_crt_sro);
+    if (src_use_speech)
+    {
+        gchar *message;
+	message = src_crt_sro ? _("added watch for current object") : 
+			_("cannot add watch for current object");
+	src_say_message (message);
+    }
+    return rv;
+}
+
 static gboolean
 src_nav_caret ()
 {
@@ -1463,7 +1481,7 @@ src_nav_statusbar_flat ()
     gboolean rv = FALSE;
     
     crt_line = screen_review_get_focused_line ();	
-    rv = src_flat_review_next_line();
+    rv = src_flat_review_line (crt_line);
 
     return rv;
 }
@@ -1616,6 +1634,9 @@ src_get_mouse_coordinates_from_sro (SRObject *sro,
 	*y = rect.y;
     }
     
+    *x = *x - 1;
+    *y = *y - 1;
+    
     return rv;
 }
 
@@ -1652,7 +1673,7 @@ src_show_x_coord()
 	gchar tmp[10];
 	sprintf (tmp, "%d", x);
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	g_free (message);
     	return TRUE;
     }
@@ -1675,7 +1696,7 @@ src_show_y_coord ()
 	gchar tmp[10];
 	sprintf (tmp, "%d", y);
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	g_free (message);
     	return TRUE;
     }
@@ -1711,7 +1732,7 @@ src_font_style ()
     	style = g_strdup ("default");
 	
     message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), style);
-    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message);
     g_free (style);
 
@@ -1746,7 +1767,7 @@ src_font_size ()
 	size = g_strdup ("default");
 	
     message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), size);
-    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message);
     g_free (size);
 
@@ -1780,7 +1801,7 @@ src_font_name ()
 	name = g_strdup ("default");
 	
     message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), name);
-    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message);
     g_free (name);
 
@@ -1813,7 +1834,7 @@ src_text_attributes ()
 	all = g_strdup ("default");
 	
     message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), all);
-    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message);
     g_free (all);
 
@@ -2092,7 +2113,7 @@ src_surroundings_logic ()
 			    message4, message5, message6, NULL);
 
     if (message)
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	
 #ifdef SRCTRL_DEBUG
     fprintf (stderr, "\n%s", message); 
@@ -2140,6 +2161,9 @@ src_overview ()
     gint i;
     gchar *role;
 
+    if (!src_use_speech)
+	return FALSE;
+
     if (!src_focused_sro)
 	return FALSE;
 
@@ -2206,7 +2230,7 @@ src_overview ()
     message = g_strconcat (message1, message2, message3, 
 			    message4, message5, message6, NULL);
 
-    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	
     g_free (message);
     g_free (message1);
@@ -2296,7 +2320,7 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 		    g_free (tmp);
 		};
 		break;
-	    case SRW_DELIMITATOR_CELL:
+	    case SRW_DELIMITER_CELL:
 		{
 		    gchar *tmp, *tmp2;
 		    tmp = brlout;
@@ -2374,7 +2398,7 @@ src_flat_review_line (gint index)
 	{
 	    gchar *message;
 	    message = src_xml_make_part ("TEXT", src_speech_get_voice ("empty"), _("empty line"));
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	    g_free (message);
 	}
 	else
@@ -2382,7 +2406,7 @@ src_flat_review_line (gint index)
 	    gchar *aux_str = NULL, *message;
 	    aux_str = g_strdup_printf (ngettext ("%d empty line", "%d empty lines", line->is_empty), line->is_empty );
 	    message = src_xml_make_part ("TEXT", src_speech_get_voice ("empty"), aux_str);
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	    g_free (message);
 	    g_free (aux_str);
 	}
@@ -2392,7 +2416,7 @@ src_flat_review_line (gint index)
 	gchar *spout;
     	spout = src_present_line_for_speech (line);
     	if (spout)
-	    src_speech_send_chunk (spout, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (spout, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	g_free (spout);
     }
     
@@ -2474,11 +2498,22 @@ src_flat_review ()
 	sro_release_reference (window);
 	n_lines = screen_review_init (&clip_rectangle, 
 				      src_crt_sro,
-				      screen_review_flag);
-					      
-	crt_line = screen_review_get_focused_line ();	
-	flat_review_x = clip_rectangle.x;
-	src_flat_review_next_line();
+				      screen_review_flag,
+				      SRW_SCOPE_WINDOW);
+	if (n_lines)
+	{				      
+	    crt_line = screen_review_get_focused_line ();	
+	    flat_review_x = clip_rectangle.x;
+	    src_flat_review_line (crt_line);
+	}
+	else
+	{
+	    if (src_use_speech)
+		src_say_message (_("cannot make flat review for current window"));
+	    sru_warning (_("cannot make flat review for current window"));
+	    src_ctrl_flat_mode_terminate ();
+	    return FALSE;
+	}
     }
     else
     {
@@ -2603,7 +2638,7 @@ src_hierarchy_logic ()
     sro_release_reference (parent);
     
     if (message)
-    	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+    	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 
     g_free (message);
     return TRUE;
@@ -2635,7 +2670,15 @@ src_hierarchy_flat ()
 			  0);
 	sro_release_reference (window);
 	
-	n_lines = cnt = screen_review_init (&clip_rectangle, src_crt_sro, SRW_ALIGNF_ALL);
+	n_lines = cnt = screen_review_init (&clip_rectangle, src_crt_sro, SRW_ALIGNF_ALL, SRW_SCOPE_WINDOW);
+	if (!n_lines)
+	{
+	    if (src_use_speech)
+		src_say_message (_("cannot make flat review for current window"));
+	    sru_warning (_("cannot make flat review for current window"));
+	    src_ctrl_flat_mode_terminate ();
+	    return FALSE;
+	}
 	flat_review_x = clip_rectangle.x;
     }
     spout = g_strdup ("");
@@ -2657,7 +2700,7 @@ src_hierarchy_flat ()
 	}
     }
     if (spout[0])
-	src_speech_send_chunk (spout, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (spout, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (spout);
     screen_review_terminate ();
 
@@ -2855,6 +2898,63 @@ src_braille_scroll_display_right ()
     return TRUE; 
 }
 
+static gboolean
+src_braille_on_off ()
+{
+    gboolean braille_on_off = !src_use_braille;
+    
+    if (!srconf_set_data (SRCORE_BRAILLE_ACTIVE,
+			  CFGT_BOOL, 
+			  &braille_on_off,
+			  SRCORE_PATH)
+        ) return FALSE;
+	
+    return TRUE;
+}
+
+static gboolean
+src_speech_on_off ()
+{
+    gboolean speech_on_off = !src_use_speech;
+    
+    if (!srconf_set_data (SRCORE_SPEECH_ACTIVE,
+			  CFGT_BOOL, 
+			  &speech_on_off,
+			  SRCORE_PATH)
+        ) return FALSE;
+
+    return TRUE;
+}
+
+static gboolean
+src_magnifier_on_off ()
+{
+    gboolean magnifier_on_off = !src_use_magnifier;
+    
+    if (!srconf_set_data (SRCORE_MAGNIF_ACTIVE,
+			  CFGT_BOOL, 
+			  &magnifier_on_off,
+			  SRCORE_PATH)
+        ) return FALSE;
+	
+    return TRUE;
+}
+
+static gboolean
+src_braille_monitor_on_off ()
+{
+    gboolean braille_monitor_on_off = !src_use_braille_monitor;
+    
+    if (!srconf_set_data (SRCORE_BRAILLE_MONITOR_ACTIVE,
+			  CFGT_BOOL, 
+			  &braille_monitor_on_off,
+			  SRCORE_PATH)
+        ) return FALSE;
+
+    return TRUE;
+}
+
+
 
 static struct
 {
@@ -2912,6 +3012,7 @@ static struct
 	{"toggle tracking mode",	src_toggle_tracking_mode	},
 
 	{"attributes at caret", 	src_caret_attributes	 	},
+	{"watch current object", 	src_watch_current	 	},
 
 	{"window hierarchy",		src_hierarchy_logic		},
 	{"window hierarchy flat",	src_hierarchy_flat		},
@@ -2962,6 +3063,11 @@ static struct
 	{"display left", 		src_braille_scroll_display_left	},
 	{"display right", 		src_braille_scroll_display_right},
 
+	{"braille on/off",		src_braille_on_off		},
+	{"speech on/off",		src_speech_on_off		},
+	{"magnifier on/off",		src_magnifier_on_off		},
+	{"braille monitor on/off", 	src_braille_monitor_on_off	}
+
     };
 
 static GQueue *src_cmd_queue 	= NULL;
@@ -3356,7 +3462,7 @@ src_ctrl_position_sensor_action_flat (gint index)
 	gchar *message;
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("role"), role);
 	if (message)
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 	g_free (message);
 	SR_freeString (role);
     }
@@ -3371,7 +3477,7 @@ src_ctrl_position_sensor_action_logic (gint index)
     if (!src_crt_sro)
 	return;
 	
-    src_text_index = index - 4 >= 0 ? index - 4 : 0;
+    src_text_index = index >= 0 ? index : 0;
 
     switch (src_braille_position_sensor)
     {
@@ -3445,7 +3551,7 @@ src_ctrl_optical_sensor_action_logic (gint index)
     if (!src_crt_sro)
 	return;
 
-    src_text_index = index - 4 >= 0 ? index - 4 : 0;
+    src_text_index = index >= 0 ? index : 0;
 
     switch (src_braille_optical_sensor)
     {
