@@ -305,7 +305,8 @@ typedef enum _SRObjectRole
     SR_ROLE_TABLE_ROW_HEADER ,
 /*    SR_ROLE_TEAROFF_MENU_ITEM ,*/
     /* An object that presents text to the user */
-    SR_ROLE_TEXT,
+    SR_ROLE_TEXT_ML,
+    SR_ROLE_TEXT_SL,
     /*
      * A specialized push button that can be checked or unchecked, but does
      * not procide a separate indicator for the current state.
