@@ -26,7 +26,7 @@
 #include <stdlib.h>
 #include "glib.h"
 
-#define SRU_PARANOIA
+/* #define SRU_PARANOIA */
 
 #ifdef SRU_PARANOIA
 #ifdef G_HAVE_ISO_VARARGS
